library(tidyverse)

args                 <- commandArgs(trailingOnly = TRUE)

modelimputations_dir <- args[[1]]
gosefile             <- args[[2]]
outputfile           <- args[[3]]

set.seed(42)

df_gose              <- readRDS(gosefile)

posteriors <- list.files(modelimputations_dir, pattern = "*.rds")

# load and combine all posteriros over different (baseline) imputations
df_imputations <- c()
for (i in 1:length(posteriors)) {
  df_imputations <- rbind(
    df_imputations,
    readRDS(sprintf("%s/%s", modelimputations_dir, posteriors[i])) %>%
      mutate(
        imp = i
      )
  )
}
df_imputations <- df_imputations %>%
  group_by(gupi, t, GOSE) %>%
  summarize(probability = mean(probability, na.rm = TRUE)) %>%
  ungroup




df_imputations <- df_imputations %>%
  select(-t) %>%
  filter(complete.cases(.)) %>%
  group_by(gupi) %>%
  mutate(
    Subject.DerivedImputed180DaysGOSE = which.max(probability) %>%
      {
        if (length(.) == 0) { # fix all probs. NA case
          return(NA)
        } else {
          return(.)
        }
      } %>%
      sample(1) %>% # need to resolve ambiguous gose predictions !
      rep(n()),
    GOSE = factor(
      GOSE,
      levels = 1:8,
      labels = sprintf("Subject.DerivedImputed180DaysGOSE_%i_probability", 1:8)
    )
  ) %>%
  ungroup %>%
  spread(GOSE, probability)

# extract per protocol gose observations
df_per_protocol_gose <- df_gose %>%
  filter(
    Outcomes.DerivedCompositeGOSEDaysPostInjury >= 5*30,
    Outcomes.DerivedCompositeGOSEDaysPostInjury <= 8*30
  ) %>%
  # pick closest to 180
  group_by(gupi) %>%
  summarize(
    closest_per_protocol_GOSE = Outcomes.DerivedCompositeGOSE[
      which.min(abs(Outcomes.DerivedCompositeGOSEDaysPostInjury - 180))
    ]
  )

df_imputations <- df_imputations %>%
  left_join(df_per_protocol_gose, by = "gupi") %>%
  mutate(
    Subject.GOSE6monthEndpointDerived = ifelse(
      is.na(closest_per_protocol_GOSE),
      Subject.DerivedImputed180DaysGOSE,
      closest_per_protocol_GOSE
    )
  ) %>%
  select(
    gupi,
    Subject.GOSE6monthEndpointDerived,
    Subject.DerivedImputed180DaysGOSE,
    everything()
  ) %>%
  select(
    -closest_per_protocol_GOSE
  )

write_csv(df_imputations, outputfile)
